#include <string>
#include "LumiverseCore.h"

using namespace std;
using namespace Lumiverse;

int main(int argc, char**argv) {
  // Creates an empty rig.
  Rig rig;

  // Create a new device
  Device* d = new Device("myFirstDevice", 1, "ETC Source Four 26deg");

  // Add a parameter to the device. Parameters are controlled over the network.
  d->setParam("intensity", new LumiverseFloat(0, 0, 100, 0));

  // Add a metadata value to the device. Metadata values are not controlled over the network.
  d->setMetadata("myData", "[some value]");

  // Add the device to the rig
  rig.addDevice(d);

  // Now to tell the rig how to communicate with the light.
  // We create a DMX patch. Almost all Patches in Lumiverse are DMX patches at the moment.
  DMXPatch* p = new DMXPatch();

  // Create a KiNet inetrface and add it to the Patch. Assign the interface to universe 1.
  p->assignInterface((DMXInterface*)(new KiNetInterface("myKiNetEnabler", "10.1.0.1")), 1);

  // Now we need to tell the patch a few things.
  // 1. How to convert the LumiverseFloat parameter to a DMX value
  // 2. Where on the network "myFirstDevice" lives.

  // Create a Device Map that tells the Rig how to translate our device to DMX values
  map<string, patchData> devProfile;
  devProfile["intensity"] = patchData(0, FLOAT_TO_SINGLE);
  // Add the map to the Patch
  p->addDeviceMap("ETC Source Four", devProfile);

  // Tell the patch how to deal with the "myFirstDevice" device
  p->patchDevice("myFirstDevice", new DMXDevicePatch("ETC Source Four", 0, 1));

  // Now we're all set
  // Initialize the rig.
  rig.init();

  // Start the rig.
  rig.run();

  // Do some stuff with lights!
  rig["myFirstDevice"]->setParam("intensity", 100);

  float val;
  rig["myFirstDevice"]->getParam("intensity", val);
  cout << "myFirstDevice Intensity: " << val << "%\n";

  // Save our configuration for easy use later
  rig.save("/path/to/myFirstRig.rig.json");

  getchar();
}